/* cmd_funcmds.c - NeonServ v5.6
 * Copyright (C) 2011-2012  Philipp Kreil (pk910)
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program. If not, see <http://www.gnu.org/licenses/>. 
 */
#include "../module.h"
#include "cmd_funcmds.h"
#include "../../modcmd.h"
#include "../../mysqlConn.h"
#include "../../IRCParser.h"
#include "../../ClientSocket.h"
#include "../../UserNode.h"
#include "../../ChanNode.h"
#include "../../lang.h"
#include "../../tools.h"
#include "../../DBHelper.h"
#include "../../IRCEvents.h"
#include "../../timeq.h"
#include "../../WHOHandler.h"
#include "../../EventLogger.h"

static const struct default_language_entry msgtab[] = {
    {"FUN_DICE", "$b%s$b: A $b%d$b shows on the %d-sided die."}, /* {ARGS: "TestUser", 5, 6} */
    {"FUN_DICE_NUM", "I do not understand $b%s$b. Please use a single number above 1."}, /* {ARGS: "bla"} */
    {"FUN_8BALL", "$b%s$b: %s"}, /* {ARGS: "TestUser", "Not a chance."} */
    {"FUN_8BALL_REPLIES", "Not a chance.|In your dreams.|Absolutely!|Could be, could be.|No!"},
    {"FUN_COOKIE", "gives %1$s a very big chocolate cookie. %1$s has got %2$d cookies until now (%3$d in this channel)."}, /* {ARGS: "TestUser", 20, 50} */
    {"FUN_BOMB_MENU", "There are the following commands:"},
    {"FUN_BOMB_MENU_PLANT",  "$b%s plant <user> [wires] $b Plant a bomb in front of another users feet ([wires] is the number of wires - default: %d)"}, /* {ARGS: "bomb", 3} */
    {"FUN_BOMB_MENU_KPLANT", "$b%s kplant <user> [wires]$b Plant a kick-bomb in front of another users feet ([wires] is the number of wires - default: %d) (OP only)"}, /* {ARGS: "bomb", 3} */
    {"FUN_BOMB_MENU_DEFUSE", "$b%s defuse <wire>        $b Defuses a bomb (maybe :D)"}, /* {ARGS: "bomb"} */
    {"FUN_BOMB_MENU_RETURN", "$b%s return               $b Pushes a bomb planted infront of you back to the owner ($k4WARNING$k: There is a highly chance that the bomb detonates)"}, /* {ARGS: "bomb"} */
    {"FUN_BOMB_SELF", "You can not plant a bomb in front of yourself..."},
    {"FUN_BOMB_TARGET_ACTIVE", "%s has already an active bomb..."},
    {"FUN_BOMB_OWNER_ACTIVE", "You have already planted another bomb..."},
    {"FUN_BOMB_PLANTED", "%1$s planted a bomb in front of %2$s's feet. %2$s, you have $b%3$d seconds$b to defuse the bomb by cutting one of the following wires: %4$s ($uuse:$u %5$s defuse <wire>)"}, /* {ARGS: "TestUser", "TestTarget", 30, "blue, red, green", "bomb"} */
    {"FUN_BOMB_KPLANTED", "%1$s planted a kick-bomb in front of %2$s's feet. %2$s, you have $b%3$d seconds$b to defuse the bomb by cutting one of the following wires: %4$s ($uuse:$u %5$s defuse <wire>)"}, /* {ARGS: "TestUser", "TestTarget", 30, "blue, red, green", "bomb"} */
    {"FUN_BOMB_PLANT_SERVICE", "You can't plant a bomb in front of this user (bot)..."},
    {"FUN_BOMB_PLANT_PROTECTED", "You can't plant a bomb in front of this user (protect)..."},
    {"FUN_BOMB_PLANT_MAXWIRES", "A bomb can only have %d-%d wires!"}, /* {ARGS: 2, 8} */
    {"FUN_BOMB_WIRES", "$k4red$k|$k3green$k|$k8yellow$k|$k12blue$k|$k1black$k|$k6purple$k|$k7orange$k|$k11aqua$k"},
    {"FUN_BOMB_NOBOMB", "There is no bomb you could defuse..."},
    {"FUN_BOMB_UNKNOWN_WIRE", "%s is an unknown wire..."}, /* {ARGS: "white"} */
    {"FUN_BOMB_DEFUSED", "%1$s has successfully defused the bomb! %1$s got %2$d bombs (%3$d in this channel) and has defused %4$d of them."}, /* {ARGS: "TestUser", 20, 10, 5} */
    {"FUN_BOMB_DETONATED", "*BOOOOOOM* The bomb explodes in front of %1$s!!! %2$s was the right wire. %1$s got %3$d bombs (%4$d in this channel)"}, /* {ARGS: "TestUser", "white", 20, 10} */
    {NULL, NULL}
};

#define FUNCMD_BOMB_WIRES_DEFAULT 3
#define FUNCMD_BOMB_WIRES_MIN     2
#define FUNCMD_BOMB_WIRES_MAX     8
#define FUNCMD_BOMB_TIME          30

static int funcmd_bomb_freeuser(struct UserNode *user);
static int funcmd_bomb_freechan(struct ChanNode *chan);
static void funcmd_bomb_freeclient(struct ClientSocket *client);

void init_funcmds() {
    register_default_language_table(msgtab);
    srand(time(NULL));
    bind_freeuser(funcmd_bomb_freeuser, module_id);
    bind_freechan(funcmd_bomb_freechan, module_id);
    bind_freeclient(funcmd_bomb_freeclient, module_id);
}

void register_commands() {
    //Fun Commands
    register_command_alias(3, "FunCMD");
    #define USER_COMMAND(NAME,FUNCTION,PARAMCOUNT,FLAGS) register_command(3, NAME, module_id, FUNCTION, PARAMCOUNT, NULL, 0, FLAGS)
    //               NAME              FUNCTION        PARAMS   FLAGS
    //USER_COMMAND("extscript",    neonserv_cmd_extscript, 0,  CMDFLAG_REQUIRE_CHAN | CMDFLAG_REGISTERED_CHAN | CMDFLAG_EMPTY_ARGS | CMDFLAG_CHAN_PARAM | CMDFLAG_FUNCMD);
    USER_COMMAND("ping",         funcmd_ping,            0,  CMDFLAG_REQUIRE_CHAN | CMDFLAG_REGISTERED_CHAN | CMDFLAG_FUNCMD);
    USER_COMMAND("pong",         funcmd_pong,            0,  CMDFLAG_REQUIRE_CHAN | CMDFLAG_REGISTERED_CHAN | CMDFLAG_FUNCMD);
    USER_COMMAND("dice",         funcmd_dice,            1,  CMDFLAG_REQUIRE_CHAN | CMDFLAG_REGISTERED_CHAN | CMDFLAG_FUNCMD);
    USER_COMMAND("8ball",        funcmd_8ball,           1,  CMDFLAG_REQUIRE_CHAN | CMDFLAG_REGISTERED_CHAN | CMDFLAG_FUNCMD);
    USER_COMMAND("cookie",       funcmd_cookie,          0,  CMDFLAG_REQUIRE_CHAN | CMDFLAG_REGISTERED_CHAN | CMDFLAG_FUNCMD);
    USER_COMMAND("bomb",         funcmd_bomb,            0,  CMDFLAG_REQUIRE_CHAN | CMDFLAG_REGISTERED_CHAN | CMDFLAG_FUNCMD);
    USER_COMMAND("ship",         funcmd_ship,            0,  CMDFLAG_REQUIRE_CHAN | CMDFLAG_REGISTERED_CHAN | CMDFLAG_FUNCMD);
    #undef USER_COMMAND
}

struct funcmd_header_info {
    struct ClientSocket *client;
    struct UserNode *user;
    struct ChanNode *chan;
    char send_notice;
    char null_language;
};

#define FUNCMD_HEADER \
struct funcmd_header_info *header = malloc(sizeof(*header)); \
header->client = textclient; \
header->user = user; \
header->chan = chan; \
header->null_language = 0; \
{\
    MYSQL_RES *res; \
    MYSQL_ROW row; \
    printf_mysql_query("SELECT `channel_toys` FROM `channels` WHERE `channel_name` = '%s'", escape_string(chan->name)); \
    res = mysql_use(); \
    row = mysql_fetch_row(res); \
    if(!row || !strcmp(row[0], "0")) { \
        reply(textclient, user, "NS_FUN_DISABLED", chan->name); \
        return; \
    } else if(!strcmp(row[0], "1")) \
        header->send_notice = 1; \
    else \
        header->send_notice = 0; \
}

#define FUNCMD_FOOTER \
free(header);

#define REPLYTYPE_NORMAL 0x01
#define REPLYTYPE_ACTION 0x02
#define REPLYTYPE_NOTICE 0x04
static void funcmd_reply(struct funcmd_header_info *header, const char *text, int type, ...) {
    if (!(header->client->flags & SOCKET_FLAG_CONNECTED)) return;
    const char *reply_format = get_language_string((header->null_language ? NULL : header->user), text);
    if(reply_format)
        text = reply_format;
    char formatBuf[MAXLEN];
    if(header->send_notice || (type & REPLYTYPE_NOTICE)) {
        if(type & REPLYTYPE_ACTION)
            sprintf(formatBuf, "NOTICE %s :%s %s", header->user->nick, header->client->user->nick, text);
        else
            sprintf(formatBuf, "NOTICE %s :%s", header->user->nick, text);
    } else {
        if(type & REPLYTYPE_ACTION)
            sprintf(formatBuf, "PRIVMSG %s :\001ACTION %s\001", header->chan->name, text);
        else
            sprintf(formatBuf, "PRIVMSG %s :%s", header->chan->name, text);
    }
    va_list arg_list;
    char sendBuf[MAXLEN];
    int pos;
    sendBuf[0] = '\0';
    va_start(arg_list, type);
    pos = vsnprintf(sendBuf, MAXLEN - 2, formatBuf, arg_list);
    va_end(arg_list);
    if (pos < 0 || pos > (MAXLEN - 2)) pos = MAXLEN - 2;
    sendBuf[pos] = '\n';
    sendBuf[pos+1] = '\0';
    write_socket(header->client, sendBuf, pos+1);
}

static char* getSetting(struct UserNode *user, struct ChanNode *chan, const char *setting) {
    char *uname = "";
    int cid = 0;
    MYSQL_RES *res;
    MYSQL_ROW row;
    if(user) {
        uname = ((user->flags & USERFLAG_ISAUTHED) ? user->auth : "*");
    }
    if(chan) {
        loadChannelSettings(chan);
        if(chan->flags & CHANFLAG_CHAN_REGISTERED)
            cid = chan->channel_id;
    }
    printf_mysql_query("SELECT `value` FROM `fundata` WHERE `user` = '%s' AND `cid` = '%d' AND `name` = '%s'", escape_string(uname), cid, escape_string(setting));
    res = mysql_use();
    if ((row = mysql_fetch_row(res)) != NULL) {
        return row[0];
    } else
        return NULL;
}

static void setSetting(struct UserNode *user, struct ChanNode *chan, const char *setting, const char *value) {
    char *uname = "";
    int cid = 0;
    MYSQL_RES *res;
    MYSQL_ROW row;
    if(user) {
        uname = ((user->flags & USERFLAG_ISAUTHED) ? user->auth : "*");
    }
    if(chan) {
        loadChannelSettings(chan);
        if(chan->flags & CHANFLAG_CHAN_REGISTERED)
            cid = chan->channel_id;
    }
    printf_mysql_query("SELECT `id`, `value` FROM `fundata` WHERE `user` = '%s' AND `cid` = '%d' AND `name` = '%s'", escape_string(uname), cid, escape_string(setting));
    res = mysql_use();
    if ((row = mysql_fetch_row(res)) != NULL) {
        if(strcmp(row[1], value))
            printf_mysql_query("UPDATE `fundata` SET `value` = '%s' WHERE `id` = '%s'", escape_string(value), row[0]);
    } else
        printf_mysql_query("INSERT INTO `fundata` (`user`, `cid`, `name`, `value`) VALUES ('%s', '%d', '%s', '%s')", escape_string(uname), cid, escape_string(setting), escape_string(value));
}

#include "cmd_funcmds_bomb.c"


CMD_BIND(funcmd_ping) {
    FUNCMD_HEADER;
    funcmd_reply(header, "\002%s\002: Pong!", REPLYTYPE_NORMAL, user->nick);
    FUNCMD_FOOTER;
}

CMD_BIND(funcmd_pong) {
    FUNCMD_HEADER;
    funcmd_reply(header, "\002%s\002: Ping!", REPLYTYPE_NORMAL, user->nick);
    FUNCMD_FOOTER;
}

CMD_BIND(funcmd_dice) {
    FUNCMD_HEADER;
    int max = atoi(argv[0]);
    if(max > 1) {
        int val = (rand() % max) + 1;
        funcmd_reply(header, "%s", REPLYTYPE_NORMAL, "     ____");
        funcmd_reply(header, "%s", REPLYTYPE_NORMAL, "    /\\' .\\    _____");
        funcmd_reply(header, "%s", REPLYTYPE_NORMAL, "   /: \\___\\  / .  /\\");
        funcmd_reply(header, "%s", REPLYTYPE_NORMAL, "   \\' / . / /____/..\\");
        funcmd_reply(header, "%s", REPLYTYPE_NORMAL, "    \\/___/  \\'  '\\  /");
        funcmd_reply(header, "%s", REPLYTYPE_NORMAL, "             \\'__'\\/ ");
        funcmd_reply(header, "FUN_DICE", REPLYTYPE_NORMAL, user->nick, val, max);
    } else
        funcmd_reply(header, "FUN_DICE_NUM", REPLYTYPE_NORMAL, argv[0]);
    FUNCMD_FOOTER;
}

CMD_BIND(funcmd_8ball) {
    FUNCMD_HEADER;
    char *message = merge_argv(argv, 0, argc);
    const char *const_replies = get_language_string(header->user, "FUN_8BALL_REPLIES");
    char replies[MAXLEN];
    int i, reply_count = 1;
    for(i = 0; const_replies[i]; i++) {
        if(const_replies[i] == '|')
            reply_count++;
        replies[i] = const_replies[i];
    }
    replies[i] = '\0';
    unsigned int crc32_val = (crc32(message)) % reply_count;
    char *creply = (crc32_val == 0 ? replies : NULL);
    reply_count = 0;
    for(i = 0; replies[i]; i++) {
        if(replies[i] == '|') {
            if(creply) {
                replies[i] = '\0';
                break;
            } else {
                reply_count++;
                if(reply_count == crc32_val) {
                    creply = &replies[i+1];
                }
            }
        }
    }
    if(creply) {
        funcmd_reply(header, "FUN_8BALL", REPLYTYPE_NORMAL, user->nick, creply);
    }
    FUNCMD_FOOTER;
}

CMD_BIND(funcmd_cookie) {
    FUNCMD_HEADER;
    if(argc) {
        if(!(user = getUserByNick(argv[0]))) {
            reply(header->client, header->user, "NS_USER_UNKNOWN", argv[0]);
            FUNCMD_FOOTER;
            return;
        }
    }
    char *tmp;
    int user_count = ((tmp = getSetting(user, chan, "cookies")) ? atoi(tmp) : 0);
    int total_count = ((tmp = getSetting(user, NULL, "cookies")) ? atoi(tmp) : 0);
    user_count++;
    total_count++;
    char buf[10];
    sprintf(buf, "%d", user_count);
    setSetting(user, chan, "cookies", buf);
    sprintf(buf, "%d", total_count);
    setSetting(user, NULL, "cookies", buf);
    funcmd_reply(header, "FUN_COOKIE", REPLYTYPE_ACTION, user->nick, total_count, user_count);
    FUNCMD_FOOTER;
}

CMD_BIND(funcmd_ship) {
    FUNCMD_HEADER;
    funcmd_reply(header, "%s", REPLYTYPE_NORMAL, "\0030,0------------------------\0031|");
    funcmd_reply(header, "%s", REPLYTYPE_NORMAL, "\0030,0----------------------\00314)_)_)");
    funcmd_reply(header, "%s%c%s", REPLYTYPE_NORMAL, "\0030,0--------------------\0031,4\002=", header->client->user->nick[0], "=\002\00314,0|)___)");
    funcmd_reply(header, "%s", REPLYTYPE_NORMAL, "\0030,0-------------------\00314)__) )___) )");
    funcmd_reply(header, "%s", REPLYTYPE_NORMAL, "\0030,0------------------\00314)___) )____))_)");
    funcmd_reply(header, "%s", REPLYTYPE_NORMAL, "\0030,0----------------\00314)____)_____))__)\\");
    funcmd_reply(header, "%s", REPLYTYPE_NORMAL, "\0030,0---------------\00314)\0035__\00314|\0035____\00314/|\0035___\00314|\0035___\00314-\\\\---");
    funcmd_reply(header, "%s", REPLYTYPE_NORMAL, "\0030,0-------\00312^^^^^^\0035~~~\\ \002oo oo oo oo\002 /~~\00312^^^^^^");
    funcmd_reply(header, "%s", REPLYTYPE_NORMAL, "\0030,0------------\00312~^^^^ ~~~~^^~~~~^^~~^^~~~~~");
    funcmd_reply(header, "%s", REPLYTYPE_NORMAL, "\0030,0-----------------\00312~~^^ ~^^~ ~^~ ~^");
    FUNCMD_FOOTER;
}
